@@ -145,17 +145,25 @@ prep() {
   unpack ${src_orig_pkg} && \
   cd ${topdir} && \
   if [ -f ${src_patch} ] ; then \
-    patch -b -Z -p0 < ${src_patch} ;\
+    patch -b -N -Z -p0 < ${src_patch} ;\
+    # workaround patch -b bug 
+    find ${srcdir} -name \*.orig | xargs chmod u+rw ;\
+    for f in ${srcdir}/CYGWIN-PATCHES/*.orig; do rm -f $f; done; \
   fi && \
   mkdirs )
 }
 fixup() {
   # shared lib patch by Brian Brubs (TBD: upstream)
+  # needed to move the LIBTOOL macros from acinclude.m4 to aclocal.m4 
+  # (or just remove them and run aclocal)
   (cd ${srcdir}
     #rm config.guess config.sub
     autoreconf -f -i
     )
 }
+# TODO: check libmilter support: http://milter.free.fr/intro/all.htm
+# only sendmail.
+# fetchmail? procmail? ssmtp? email? nail? uw-mail?
 conf() {
 # --disable-pthreads
   (cd ${objdir} && \
@@ -179,12 +187,11 @@ reconf() {
   conf )
 }
 # extra package clamav-contrib for the windows UI?
-# current problem: libtool thinks that cygclamav*.dll is an exe. it omits -shared from the linker line.
 build() {
   (cd ${objdir}
   #LDFLAGS="-Wl,--enable-runtime-pseudo-reloc ${MY_LDFLAGS}" 
   make CFLAGS="${MY_CFLAGS}" | tee ${srcinstdir}/make.log 2>&1 
-#  LDFLAGS="-Wl,--enable-runtime-pseudo-reloc ${MY_LDFLAGS}" make -c libclamav CFLAGS="${MY_CFLAGS}" | tee -a ${srcinstdir}/make.log 2>&1 )
+#  LDFLAGS="-Wl,--enable-runtime-pseudo-reloc ${MY_LDFLAGS}" make -C clamav-milter CFLAGS="${MY_CFLAGS}" | tee -a ${srcinstdir}/make.log 2>&1
   )
 #  (cd ${srcdir} && \
 #  CFLAGS="${MY_CFLAGS}" make contrib shared )
@@ -332,7 +339,7 @@ pkg() {
 # missing: preremove, permissions, check if in use.
 setup() {
   (tar -C / -jxvf ${FULLPKG}.tar.bz2 && \
-  post="/etc/postinstall/${PKG}.sh" \
+  post="/etc/postinstall/${PKG}.sh"; \
   test -f ${post} && ${post} && mv ${post} ${post}.done )
 }
 difforig() {
@@ -466,7 +473,7 @@ while test -n "$1" ; do
     all)		checksig && prep && conf && build && install && \
 			strip && pkg && spkg && finish ; \
 			STATUS=$? ;;
-    *) echo "Error: bad arguments" ; exit 1 ;;
+    *) echo "Error: bad argument $1" ; exit 1 ;;
   esac
   ( exit ${STATUS} ) || exit ${STATUS}
   shift
