@@ -48,29 +48,28 @@
 #include "manager.h"
 #include "treewalk.h"
 #include "shared.h"
-#include "mbox.h"
 #include "str.h"
 #include "strrcpy.h"
 #include "memory.h"
 #include "output.h"
-#include "cfgparser.h"
+#include "misc.h"
+#include "../libclamav/others.h"
 
 #ifdef C_LINUX
 dev_t procdev;
 #endif
 
+
 int scanmanager(const struct optstruct *opt)
 {
 	mode_t fmode;
-	int ret = 0, compression = 0, fmodeint;
+	int ret = 0, compression = 0, fmodeint, options = 0;
 	struct cl_node *trie = NULL;
 	struct cl_limits *limits = NULL;
 	struct passwd *user = NULL;
 	struct stat sb;
 	char *fullpath = NULL, cwd[1024];
-	struct cfgstruct *copt, *cpt;
-	struct cl_cvd *d1, *d2;
-	const char *dbdir;
+
 
 /* njh@bandsman.co.uk: BeOS */
 #if !defined(C_CYGWIN) && !defined(C_BEOS)
@@ -110,42 +109,12 @@ int scanmanager(const struct optstruct *opt)
 	}
 
     } else {
-	/* try to find fresh directory */
-	dbdir = cl_retdbdir();
-	if((copt = parsecfg(CONFDIR"/clamav.conf", 0))) {
-	    if((cpt = cfgopt(copt, "DatabaseDirectory")) || (cpt = cfgopt(copt, "DataDirectory"))) {
-		if(strcmp(cl_retdbdir(), cpt->strarg)) {
-			char *daily = (char *) mmalloc(strlen(cpt->strarg) + strlen(cl_retdbdir()) + 15);
-		    sprintf(daily, "%s/daily.cvd", cpt->strarg);
-		    if((d1 = cl_cvdhead(daily))) {
-			sprintf(daily, "%s/daily.cvd", cl_retdbdir());
-			if((d2 = cl_cvdhead(daily))) {
-			    free(daily);
-			    if(d1->version > d2->version)
-				dbdir = cpt->strarg;
-			    else
-				dbdir = cl_retdbdir();
-			    cl_cvdfree(d2);
-			} else {
-			    free(daily);
-			    dbdir = cpt->strarg;
-			}
-			cl_cvdfree(d1);
-		    } else {
-			free(daily);
-			dbdir = cl_retdbdir();
-		    }
-		}
-	    }
-	}
 
-	if((ret = cl_loaddbdir(dbdir, &trie, &claminfo.signs))) {
+	if((ret = cl_loaddbdir(freshdbdir(), &trie, &claminfo.signs))) {
 	    mprintf("@%s\n", cl_strerror(ret));
 	    return 50;
 	}
 
-	if(copt)
-	    freecfg(copt);
     }
 
 
@@ -154,8 +123,7 @@ int scanmanager(const struct optstruct *opt)
 	return 50;
     }
 
-    /* build the proper trie */
-    if((ret=cl_buildtrie(trie)) != 0) {
+    if((ret = cl_build(trie)) != 0) {
 	mprintf("@Database initialization error: %s\n", cl_strerror(ret));;
 	return 50;
     }
@@ -192,6 +160,46 @@ int scanmanager(const struct optstruct *opt)
     else
         limits->maxratio = 200;
 
+    /* set options */
+
+    if(optl(opt, "disable-archive") || optl(opt, "no-archive"))
+	options &= ~CL_SCAN_ARCHIVE;
+    else
+	options |= CL_SCAN_ARCHIVE;
+
+    if(optl(opt, "detect-broken"))
+	options |= CL_SCAN_BLOCKBROKEN;
+
+    if(optl(opt, "block-encrypted"))
+	options |= CL_SCAN_BLOCKENCRYPTED;
+
+    if(optl(opt, "block-max"))
+	options |= CL_SCAN_BLOCKMAX;
+
+    if(optl(opt, "no-pe"))
+	options &= ~CL_SCAN_PE;
+    else
+	options |= CL_SCAN_PE;
+
+    if(optl(opt, "no-ole2"))
+	options &= ~CL_SCAN_OLE2;
+    else
+	options |= CL_SCAN_OLE2;
+
+    if(optl(opt, "no-html"))
+	options &= ~CL_SCAN_HTML;
+    else
+	options |= CL_SCAN_HTML;
+
+    if(optl(opt, "no-mail")) {
+	options &= ~CL_SCAN_MAIL;
+    } else {
+	options |= CL_SCAN_MAIL;
+
+	if(optl(opt, "mail-follow-urls"))
+	    options |= CL_SCAN_MAILURL;
+    }
+
 #ifdef C_LINUX
     if(stat("/proc", &sb) == -1)
 	procdev = 0;
@@ -207,71 +215,10 @@ int scanmanager(const struct optstruct *opt)
 	    mprintf("@Can't get absolute pathname of current working directory.\n");
 	    ret = 57;
 	} else
-	    ret = scandirs(cwd, trie, user, opt, limits);
+	    ret = scandirs(cwd, trie, user, opt, limits, options);
 
     } else if(!strcmp(opt->filename, "-")) { /* read data from stdin */
-	/*
-	 * njh@bandsman.co.uk: treat the input as a mailbox, the program
-	 * can then be used as a filter called when mail is received
-	 */
-	if(optc(opt, 'm')) {
-		const char *tmpdir;
-		char *dir;
-
-		/* njh@bandsman.co.uk: BeOS */
-#if !defined(C_CYGWIN) && !defined(C_BEOS)
-		if(!geteuid()) {
-		    if((user = getpwnam(UNPUSER)) == NULL) {
-			mprintf("@Can't get information about user %s\n", UNPUSER);
-			exit(60); /* this is critical problem, so we just exit here */
-		    }
-		}
-#endif
-
-		tmpdir = getenv("TMPDIR");
-
-		if(tmpdir == NULL)
-#ifdef P_tmpdir
-			tmpdir = P_tmpdir;
-#else
-			tmpdir = "/tmp";
-#endif
-
-		if(checkaccess(tmpdir, UNPUSER, W_OK) != 1) {
-			mprintf("@Can't write to the temporary directory.\n");
-			exit(64);
-		}
-		/* generate the temporary directory */
-
-		dir = cl_gentemp(tmpdir);
-		if(mkdir(dir, 0700)) {
-			mprintf("@Can't create the temporary directory %s\n", dir);
-			exit(63); /* critical */
-		}
-
-		if(user)
-			chown(dir, user->pw_uid, user->pw_gid);
-
-		/*
-		 * Extract the attachments into the temporary directory
-		 */
-		ret = cl_mbox(dir, 0);
-
-		if(ret == 0) {
-			/* fix permissions of extracted files */
-			fixperms(dir);
-
-			if(ret == 0) /* execute successful */
-				ret = treewalk(dir, trie, user, opt, limits);
-
-			/* remove the directory - as clamav */
-			clamav_rmdirs(dir);
-
-			/* free dir - it's not necessary now */
-			free(dir);
-		}
-	} else
-	    ret = checkstdin(trie, limits);
+	ret = checkstdin(trie, limits, options);
 
     } else {
 	int x;
@@ -288,6 +235,7 @@ int scanmanager(const struct optstruct *opt)
 		    /* we need to complete the path */
 		    if(!getcwd(cwd, sizeof(cwd))) {
 			mprintf("@Can't get absolute pathname of current working directory.\n");
+			free(limits);
 			return 57;
 		    } else {
 			fullpath = mcalloc(512, sizeof(char));
@@ -303,11 +251,11 @@ int scanmanager(const struct optstruct *opt)
 
 		switch(fmode & S_IFMT) {
 		    case S_IFREG:
-			ret = scanfile(fullpath, trie, user, opt, limits);
+			ret = scanfile(fullpath, trie, user, opt, limits, options);
 			break;
 
 		    case S_IFDIR:
-			ret = scandirs(fullpath, trie, user, opt, limits);
+			ret = scandirs(fullpath, trie, user, opt, limits, options);
 			break;
 
 		    default:
@@ -325,7 +273,7 @@ int scanmanager(const struct optstruct *opt)
     }
 
     /* free the trie */
-    cl_freetrie(trie);
+    cl_free(trie);
 
     free(limits);
 
@@ -361,9 +309,9 @@ int match_regex(const char *filename, const char *pattern)
 #endif
 }
 
-int scanfile(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits)
+int scanfile(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits, int options)
 {
-	int ret, options = 0, included;
+	int ret, included;
 	struct optnode *optnode;
 	char *argument;
 #ifdef C_LINUX
@@ -414,21 +362,14 @@ int scanfile(const char *filename, struct cl_node *root, const struct passwd *us
 	return 0;
     }
 
-    if(optl(opt, "disable-archive") || optl(opt, "no-archive"))
-	options &= ~CL_ARCHIVE;
-    else
-	options |= CL_ARCHIVE;
+    if(geteuid())
+	if(checkaccess(filename, NULL, R_OK) != 1) {
+	    if(!printinfected)
+		mprintf("%s: Access denied.\n", filename);
+	    return 0;
+	}
 
-    if(optl(opt, "block-encrypted"))
-	options |= CL_ENCRYPTED;
-
-    if(optl(opt, "no-ole2"))
-	options &= ~CL_OLE2;
-    else
-	options |= CL_OLE2;
-
-    if(optc(opt, 'm'))
-	options |= CL_MAIL;
+    claminfo.files++;
 
     /* 
      * check the extension  - this is a special case, normally we don't need to
@@ -437,7 +378,7 @@ int scanfile(const char *filename, struct cl_node *root, const struct passwd *us
      * external (if provided) when internal cannot extract data.
      */
 
-    if((cli_strbcasestr(filename, ".zip") || cli_strbcasestr(filename, ".rar")) && (options & CL_ARCHIVE)) {
+    if((cli_strbcasestr(filename, ".zip") || cli_strbcasestr(filename, ".rar")) && (options & CL_SCAN_ARCHIVE)) {
 	/* try to use internal archivers */
 	if((ret = checkfile(filename, root, limits, options)) == CL_VIRUS) {
 	    if(optl(opt, "remove")) {
@@ -457,8 +398,7 @@ int scanfile(const char *filename, struct cl_node *root, const struct passwd *us
 	} else if(ret == CL_CLEAN)
 	    return 0;
 	/* in other case try to continue with external archivers */
-	options &= ~CL_ARCHIVE; /* and disable decompression for the below checkfile() */
-	claminfo.files--; /* don't count it */
+	options &= ~CL_SCAN_ARCHIVE; /* and disable decompression for the below checkfile() */
     }
 
     if((cli_strbcasestr(filename, ".zip") && optl(opt, "unzip"))
@@ -493,20 +433,14 @@ int scanfile(const char *filename, struct cl_node *root, const struct passwd *us
 			    return 0;
 			}
 
-		    return(scandenied(filename, root, user, opt, limits));
+		    return(scandenied(filename, root, user, opt, limits, options));
 		}
 		return 0;
 	    case 1:
-		return(scancompressed(filename, root, user, opt, limits));
+		return(scancompressed(filename, root, user, opt, limits, options));
 	}
     }
 
-    if(geteuid())
-	if(checkaccess(filename, NULL, R_OK) != 1) {
-	    if(!printinfected)
-		mprintf("%s: Access denied.\n", filename);
-	    return 0;
-	}
 
     if((ret = checkfile(filename, root, limits, options)) == CL_VIRUS) {
 	if(optl(opt, "remove")) {
@@ -525,7 +459,7 @@ int scanfile(const char *filename, struct cl_node *root, const struct passwd *us
 }
 
 /* it has guaranteed read access to the archive */
-int scancompressed(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits)
+int scancompressed(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits, int options)
 {
 	int ret = 0;
 	char *tmpdir, *gendir, *userprg;
@@ -556,7 +490,7 @@ int scancompressed(const char *filename, struct cl_node *root, const struct pass
 
     /* generate the temporary directory */
 
-    gendir = cl_gentemp(tmpdir);
+    gendir = cli_gentemp(tmpdir);
     if(mkdir(gendir, 0700)) {
 	mprintf("@Can't create the temporary directory %s\n", gendir);
 	exit(63); /* critical */
@@ -568,7 +502,11 @@ int scancompressed(const char *filename, struct cl_node *root, const struct pass
 
     /* unpack file  - as unprivileged user */
     if(cli_strbcasestr(filename, ".zip")) {
-	char *args[] = { "unzip", "-P", "clam", "-o", (char *) filename, NULL };
+	char *args[] = { "unzip", "-P", "clam", "-o", NULL, NULL };
+	/* Sun's SUNWspro C compiler doesn't allow direct initialisation
+	 * with a variable
+	 */
+	args[4] = (char *) filename;
 
 	if((userprg = getargl(opt, "unzip")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
@@ -576,56 +514,64 @@ int scancompressed(const char *filename, struct cl_node *root, const struct pass
 	    ret = clamav_unpack("unzip", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".rar")) { 
-	char *args[] = { "unrar", "x", "-p-", "-y", (char *) filename, NULL };
+	char *args[] = { "unrar", "x", "-p-", "-y", NULL, NULL };
+	args[4] = (char *) filename;
 	if((userprg = getargl(opt, "unrar")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("unrar", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".arj")) { 
-        char *args[] = { "arj", "x","-y", (char *) filename, NULL };
+        char *args[] = { "arj", "x","-y", NULL, NULL };
+	args[3] = (char *) filename;
         if((userprg = getargl(opt, "arj")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("arj", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".zoo")) { 
-	char *args[] = { "unzoo", "-x","-j","./", (char *) filename, NULL };
+	char *args[] = { "unzoo", "-x","-j","./", NULL, NULL };
+	args[4] = (char *) filename;
 	if((userprg = getargl(opt, "unzoo")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("unzoo", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".jar")) { 
-	char *args[] = { "unzip", "-P", "clam", "-o", (char *) filename, NULL };
+	char *args[] = { "unzip", "-P", "clam", "-o", NULL, NULL };
+	args[4] = (char *) filename;
 	if((userprg = getargl(opt, "jar")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("unzip", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".lzh")) { 
-	char *args[] = { "lha", "xf", (char *) filename, NULL };
+	char *args[] = { "lha", "xf", NULL, NULL };
+	args[2] = (char *) filename;
 	if((userprg = getargl(opt, "lha")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("lha", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".tar")) { 
-	char *args[] = { "tar", "-xpvf", (char *) filename, NULL };
+	char *args[] = { "tar", "-xpvf", NULL, NULL };
+	args[2] = (char *) filename;
 	if((userprg = getargl(opt, "tar")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("tar", args, gendir, user, opt);
 
     } else if(cli_strbcasestr(filename, ".deb")) { 
-	char *args[] = { "ar", "x", (char *) filename, NULL };
+	char *args[] = { "ar", "x", NULL, NULL };
+	args[2] = (char *) filename;
 	if((userprg = getargl(opt, "deb")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
 	    ret = clamav_unpack("ar", args, gendir, user, opt);
 
     } else if((cli_strbcasestr(filename, ".tar.gz") || cli_strbcasestr(filename, ".tgz"))) {
-	char *args[] = { "tar", "-zxpvf", (char *) filename, NULL };
+	char *args[] = { "tar", "-zxpvf", NULL, NULL };
+	args[2] = (char *) filename;
 	if((userprg = getargl(opt, "tgz")))
 	    ret = clamav_unpack(userprg, args, gendir, user, opt);
 	else
@@ -636,7 +582,7 @@ int scancompressed(const char *filename, struct cl_node *root, const struct pass
     fixperms(gendir);
 
     if(!ret) /* execute successful */
-	ret = treewalk(gendir, root, user, opt, limits);
+	ret = treewalk(gendir, root, user, opt, limits, options);
 
     /* remove the directory  - as clamav */
     clamav_rmdirs(gendir);
@@ -719,7 +665,7 @@ int scancompressed(const char *filename, struct cl_node *root, const struct pass
     }
 }
 
-int scandenied(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits)
+int scandenied(const char *filename, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits, int options)
 {
 	char *tmpdir, *gendir, *tmpfile, *pt;
 	struct stat statbuf;
@@ -749,7 +695,7 @@ int scandenied(const char *filename, struct cl_node *root, const struct passwd *
     }
 
     /* generate the temporary directory */
-    gendir = cl_gentemp(tmpdir);
+    gendir = cli_gentemp(tmpdir);
     if(mkdir(gendir, 0700)) {
 	mprintf("@Can't create the temporary directory %s\n", gendir);
 	exit(63); /* critical */
@@ -777,7 +723,7 @@ int scandenied(const char *filename, struct cl_node *root, const struct passwd *
 	chown(tmpfile, user->pw_uid, user->pw_gid);
     }
 
-    if((ret = treewalk(gendir, root, user, opt, limits)) == 1) {
+    if((ret = treewalk(gendir, root, user, opt, limits, options)) == 1) {
 	logg("(Real infected archive: %s)\n", filename);
 	mprintf("(Real infected archive: %s)\n", filename);
 
@@ -803,9 +749,9 @@ int scandenied(const char *filename, struct cl_node *root, const struct passwd *
     return ret;
 }
 
-int scandirs(const char *dirname, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits)
+int scandirs(const char *dirname, struct cl_node *root, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits, int options)
 {
-	return treewalk(dirname, root, user, opt, limits);
+	return treewalk(dirname, root, user, opt, limits, options);
 }
 
 int checkfile(const char *filename, const struct cl_node *root, const struct cl_limits *limits, int options)
@@ -813,13 +759,14 @@ int checkfile(const char *filename, const struct cl_node *root, const struct cl_
 	int fd, ret;
 	const char *virname;
 
+
+    mprintf("*Scanning %s\n", filename);
+
     if((fd = open(filename, O_RDONLY)) == -1) {
 	mprintf("@Can't open file %s\n", filename);
 	return 54;
     }
 
-    claminfo.files++;
-
     if((ret = cl_scandesc(fd, &virname, &claminfo.blocks, root, limits, options)) == CL_VIRUS) {
 	mprintf("%s: %s FOUND\n", filename, virname);
 	logg("%s: %s FOUND\n", filename, virname);
@@ -839,15 +786,45 @@ int checkfile(const char *filename, const struct cl_node *root, const struct cl_
     return ret;
 }
 
-int checkstdin(const struct cl_node *root, const struct cl_limits *limits)
+int checkstdin(const struct cl_node *root, const struct cl_limits *limits, int options)
 {
 	int ret;
-	const char *virname;
+	const char *virname, *tmpdir;
+	char *file, buff[FILEBUFF];
+	FILE *fs;
 
 
+    /* check write access */
+    tmpdir = getenv("TMPDIR");
+
+    if(tmpdir == NULL)
+#ifdef P_tmpdir
+	tmpdir = P_tmpdir;
+#else
+	tmpdir = "/tmp";
+#endif
+
+    if(checkaccess(tmpdir, UNPUSER, W_OK) != 1) {
+	mprintf("@Can't write to temporary directory.\n");
+	return 64;
+    }
+
+    file = cli_gentemp(tmpdir);
+
+    if(!(fs = fopen(file, "wb"))) {
+	mprintf("@Can't open %s for writing\n", file);
+	return 63;
+    }
+
+    while((ret = fread(buff, 1, FILEBUFF, stdin)))
+	fwrite(buff, 1, ret, fs);
+
+    fclose(fs);
+
+    mprintf("*Checking %s\n", file);
     claminfo.files++;
 
-    if((ret = cl_scandesc(0, &virname, &claminfo.blocks, root, limits, CL_RAW)) == CL_VIRUS) {
+    if((ret = cl_scanfile(file, &virname, &claminfo.blocks, root, limits, options)) == CL_VIRUS) {
 	mprintf("stdin: %s FOUND\n", virname);
 	claminfo.ifiles++;
 
@@ -861,6 +838,8 @@ int checkstdin(const struct cl_node *root, const struct cl_limits *limits)
 	if(!printinfected)
 	    mprintf("stdin: %s\n", cl_strerror(ret));
 
+    unlink(file);
+    free(file);
     return ret;
 }
 
