@@ -21,16 +21,10 @@
 
 #include "clamav.h"
 
-struct nodelist {
-    struct cl_node *node;
-    struct nodelist *next;
-};
+int cli_scandesc(int desc, const char **virname, long int *scanned, const struct cl_node *root, short otfrec, unsigned short ftype);
 
-int cli_addpatt(struct cl_node *root, struct cli_patt *pattern);
-struct nodelist *cli_bfsadd(struct nodelist *bfs, struct cl_node *n);
-void cli_failtrans(struct cl_node *root);
-void cli_fasttrie(struct cl_node *n, struct cl_node *root);
-int cli_findpos(const char *buffer, int offset, int length, const struct cli_patt *pattern);
-int cli_scanbuff(const char *buffer, unsigned int length, const char **virname, const struct cl_node *root, int *partcnt);
+int cli_scanbuff(const char *buffer, unsigned int length, const char **virname, const struct cl_node *root, unsigned short ftype);
+
+int cli_validatesig(unsigned short target, unsigned short ftype, const char *offstr, unsigned long int fileoff, int desc, const char *virname);
 
 #endif
