@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999-2002 Tomasz Kojm <zolw@konarski.edu.pl>
+ *  Copyright (C) 1999 - 2004 Tomasz Kojm <tk@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,11 +16,12 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef __OTHERS_H
-#define __OTHERS_H
+#ifndef __OTHERS_H_LC
+#define __OTHERS_H_LC
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "cltypes.h"
 
 void cli_warnmsg(const char *str, ...);
 void cli_errmsg(const char *str, ...);
@@ -29,8 +30,13 @@ void *cli_malloc(size_t nmemb);
 void *cli_calloc(size_t nmemb, size_t size);
 void *cli_realloc(void *ptr, size_t size);
 int cli_rmdirs(const char *dirname);
-char *cli_md5stream(FILE *fd);
+char *cli_md5stream(FILE *fs, unsigned char *digcpy);
+char *cli_md5file(const char *filename);
 int cli_readn(int fd, void *buff, unsigned int count);
 int cli_writen(int fd, void *buff, unsigned int count);
+int32_t cli_readint32(const char *buff);
+char *cli_gentemp(const char *dir);
+unsigned int cli_rndnum(unsigned int max);
+int cli_memstr(const char *haystack, int hs, const char *needle, int ns);
 
 #endif
