@@ -16,13 +16,11 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* some things may need to be tuned here (look at jmp variables) */
 
 #if HAVE_CONFIG_H
 #include "clamav-config.h"
 #endif
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,6 +29,8 @@
 #include <time.h>
 #include <locale.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
@@ -48,11 +48,14 @@
 #include "memory.h"
 #include "output.h"
 #include "cfgparser.h"
+#include "misc.h"
+#include "../clamscan/others.h"
+#include "../libclamav/others.h"
+#include "../libclamav/str.h"
 
-#define LINE 1024
-
-#define MIN_LENGTH 15
-#define MAX_LENGTH 200
+#ifndef	O_BINARY
+#define	O_BINARY	0
+#endif
 
 void help(void);
 char *getdsig(const char *host, const char *user, const char *data);
@@ -62,136 +65,22 @@ int unpack(struct optstruct *opt);
 int listdb(const char *filename);
 int listdir(const char *dirname);
 void listsigs(struct optstruct *opt);
-int cli_rmdirs(const char *dirname); /* libclamav's internal */
 
-int scanfile(const char *cmd, const char *str, const char *file)
-{
-	FILE *pd;
-	char *command, buffer[LINE];
-
-
-    /* build the command */
-    command = (char *) mcalloc(strlen(cmd) + strlen(file) + 10, sizeof(char));
-    sprintf(command, "%s %s", cmd, file);
-
-    if((pd = popen(command, "r")) == NULL) {
-	mprintf("!popen() failed\n");
-	return 3;
-    }
-
-    while(fgets(buffer, LINE, pd)) {
-	if(strstr(buffer, str)) {
-	    free(command);
-            fclose(pd);
-	    return 1; /* found */
-	}
-    }
-
-    free(command);
-    fclose(pd);
-    return 0; /* substring not found */
-}
-
-char *cut(const char *file, long int start, long int end)
-{
-	char *fname = NULL, buffer[FILEBUFF];
-	int bytes, size, sum;
-	FILE *rd, *wd;
-
-
-    if((rd = fopen(file, "rb")) == NULL) {
-	mprintf("!File %s doesn't exist.\n", file);
-	exit(13);
-    }
-
-    if((fname = cl_gentemp(".")) == NULL) {
-	mprintf("!Can't generate temporary file name.\n");
-	exit(1);
-    }
-
-    if((wd = fopen(fname, "wb")) == NULL) {
-	mprintf("!Can't create temporary file %s\n", fname);
-	exit(14);
-    }
-
-    fseek(rd, start, SEEK_SET);
-
-    size = end - start;
-    sum = 0;
-
-    while((bytes = fread(buffer, 1, FILEBUFF, rd)) > 0) {
-	if(sum + bytes >= size) {
-	    fwrite(buffer, 1, size - sum, wd);
-	    break;
-	} else
-	    fwrite(buffer, 1, bytes, wd);
-
-	sum += bytes;
-    }
-
-    fclose(rd);
-    fclose(wd);
-
-    return fname;
-}
-
-char *change(const char *file, long int x)
-{
-	char *fname = NULL, buffer[FILEBUFF];
-	int bytes, ch;
-	FILE *rd, *wd;
-
-
-    if((rd = fopen(file, "rb")) == NULL) {
-	mprintf("!File %s doesn't exist.\n", file);
-	exit(13);
-    }
-
-    if((fname = cl_gentemp(".")) == NULL) {
-	mprintf("!Can't generate temporary file name.\n");
-	exit(1);
-    }
-
-    if((wd = fopen(fname, "wb+")) == NULL) {
-	mprintf("!Can't create temporary file %s\n", fname);
-	exit(14);
-    }
-
-    while((bytes = fread(buffer, 1, FILEBUFF, rd)) > 0)
-	fwrite(buffer, 1, bytes, wd);
-
-    fclose(rd);
-
-    if(x) { /* don't alter first character in the file */
-	fflush(wd);
-	fseek(wd, x, SEEK_SET);
-	ch = fgetc(wd);
-	fseek(wd, -1, SEEK_CUR);
-	fputc(++ch, wd);
-    }
-
-    fclose(wd);
-    return fname;
-}
 
 void sigtool(struct optstruct *opt)
 {
-	    char buffer[FILEBUFF];
-	    int bytes;
-	    char *pt;
 
+    if(optl(opt, "quiet"))
+	mprintf_quiet = 1;
 
-    if(optl(opt, "quiet")) mprintf_quiet = 1;
-    else mprintf_quiet = 0;
-
-    if(optl(opt, "stdout")) mprintf_stdout = 1;
-    else mprintf_stdout = 0;
+    if(optl(opt, "stdout"))
+	mprintf_stdout = 1;
 
     if(optl(opt, "debug"))
 	cl_debug();
 
     if(optc(opt, 'V')) {
-	mprintf("sigtool / ClamAV version "VERSION"\n");
+	print_version();
 	exit(0);
     }
 
@@ -201,13 +90,61 @@ void sigtool(struct optstruct *opt)
     }
 
     if(optl(opt, "hex-dump")) {
+	    char buffer[FILEBUFF];
+	    int bytes;
+	    char *pt;
 
 	while((bytes = read(0, buffer, FILEBUFF)) > 0) {
-	    pt = cl_str2hex(buffer, bytes);
+	    pt = cli_str2hex(buffer, bytes);
 	    write(1, pt, 2 * bytes);
 	    free(pt);
 	}
 
+    } else if(optl(opt, "md5")) {
+	    char *md5, *filename;
+	    int i;
+	    struct stat sb;
+
+	mprintf_stdout = 1;
+
+	if(opt->filename) {
+
+	    for(i = 0; (filename = cli_strtok(opt->filename, i, "\t")); i++) {
+		if(stat(filename, &sb) == -1) {
+		    mprintf("!Can't access file %s\n", filename);
+		    perror(filename);
+		} else {
+		    if((sb.st_mode & S_IFMT) == S_IFREG) {
+			if((md5 = cli_md5file(filename))) {
+			    mprintf("%s:%d:%s\n", md5, sb.st_size, filename);
+			    free(md5);
+			} else
+			    mprintf("!Can't generate MD5 checksum for %s\n", filename);
+		    }
+		}
+
+		free(filename);
+	    }
+
+	} else {
+
+	    md5 = cli_md5stream(stdin, NULL);
+	    mprintf("%s\n", md5);
+	    free(md5);
+	}
+
+    } else if(optl(opt, "html-normalise")) {
+	    int fd;
+
+	if((fd = open(getargl(opt, "html-normalise"), O_RDONLY)) == -1) {
+	    mprintf("Can't open file %s\n", getargl(opt, "html-normalise"));
+	    exit(1);
+	}
+
+	html_normalise_fd(fd, ".", NULL);
+
+	close(fd);
+
     } else if(optc(opt, 'b')) {
 	if(!optl(opt, "server")) {
 	    mprintf("!--server is required in this mode\n");
@@ -233,233 +170,10 @@ void sigtool(struct optstruct *opt)
 	listsigs(opt);
 
     } else {
-	    int jmp, lastjmp = 0, end, found = 0, exec = 0, pos, filesize,
-		maxsize = 0, ret;
-	    char *c, *s, *f, *tmp, *signame, *bsigname, *f2 = NULL;
-	    FILE *fd, *wd;
 
-	if(!optc(opt, 'c')) {
-	    mprintf("!--command, -c is required in this mode\n");
-	    exit(10);
-	} else if(!optc(opt, 's')) {
-	    mprintf("!--string, -s is required in this mode\n");
-	    exit(10);
-	} else if(!optc(opt, 'f')) {
-	    mprintf("!--file, -f is required in this mode\n");
-	    exit(10);
-	}
-
-	/* these are pointers to corresponding strings in option list */
-	c = getargc(opt, 'c');
-	s = getargc(opt, 's');
-	f = getargc(opt, 'f');
-
-	if(scanfile(c, s, f) != 1) {
-	    mprintf("!String %s not found in scanner's output.\n", s);
-	    mprintf("Please check it and try again.\n");
-	    mprintf("Does the scanner write to stdout ? It has to.\n");
-	    exit(11);
-	}
-
-	/* initial values */
-	filesize = end = fileinfo(f, 1);
-	jmp = end / 5 + 1;
-
-	/* find signature end */
-	while(1) {
-	    tmp = cut(f, 0, end);
-	    exec++;
-	    ret = scanfile(c, s, tmp);
-	    unlink(tmp);
-	    free(tmp);
-
-	    if(ret == 1) {
-
-		if(end >= jmp) {
-		    mprintf("Detected, decreasing end %d -> %d\n", end, end - jmp);
-		    end -= jmp;
-		} else
-		    end = 0;
-
-	    } else {
-		mprintf("Not detected at %d, moving forward.\n", end);
-		if(jmp == 1) {
-
-		    while(end <= filesize) {
-			tmp = cut(f, 0, end);
-			exec++;
-			if(scanfile(c, s, tmp) == 1) {
-			    mprintf(" *** Signature end found at %d\n", end);
-			    found = 1;
-			    f2 = strdup(tmp); /* remember this file */
-			    free(tmp);
-			    break;
-			} else {
-			    unlink(tmp);
-			    free(tmp);
-			    mprintf("Increasing end %d -> %d\n", end, end + 1);
-			}
-			end++;
-		    }
-
-		    if(found) break;
-		}
-
-		if(jmp)
-		    jmp--;
-		jmp = jmp/2 + 1;
-		end += jmp;
-		if(end > filesize)
-		    end = filesize;
-
-	    }
-
-	}
-
-	/* find signature start */
-	found = 0;
-	jmp = 50;
-	pos = end - jmp;
-
-	while(1) {
-
-	    tmp = change(f2, pos);
-	    if(scanfile(c, s, tmp) != 1) {
-		exec++;
-		unlink(tmp);
-		free(tmp);
-
-		if(pos >= jmp) {
-		    mprintf("Not detected, moving backward %d -> %d\n", pos, pos - jmp);
-		    pos -= jmp;
-		    maxsize += jmp;
-		} else {
-		    mprintf("Not detected, using the beginning of the file.\n");
-		    pos = 0;
-		    break;
-		}
-
-		if(maxsize > MAX_LENGTH) {
-		    mprintf("!Generated signature is too big.\n");
-		    unlink(f2);
-		    free(f2);
-		    exit(1);
-		}
-
-	    } else {
-		mprintf("Detected at %d, moving forward.\n", pos);
-		if(jmp == 1 && lastjmp == 1) {
-		    unlink(tmp);
-		    free(tmp);
-		    while(pos < end) {
-			tmp = change(f2, pos);
-			exec++;
-			ret = scanfile(c, s, tmp);
-			unlink(tmp);
-			free(tmp);
-			if(ret == 1) {
-			    mprintf("Moving forward %d -> %d\n", pos, pos + 1);
-			    pos++;
-
-			    if(end - pos < MIN_LENGTH) {
-				mprintf("!Generated signature is too small.\n");
-				unlink(f2);
-				free(f2);
-				exit(1);
-			    }
-
-			} else {
-			    mprintf(" *** Signature start found at %d\n", pos);
-			    found = 1;
-			    break;
-			}
-		    }
-
-		    if(pos >= end) {
-		        mprintf("!Can't generate a proper signature.\n");
-			unlink(f2);
-			free(f2);
-		        exit(1);
-		    }
-
-		    if(found)
-			break;
-		}
-
-		lastjmp = jmp;
-		if(jmp > 0)
-		    jmp--;
-		jmp = jmp/2 + 1;
-		pos += jmp;
-
-		if(pos >= end - 2 * jmp)
-		    pos = end - 2 * jmp;
-
-		unlink(tmp);
-		free(tmp);
-	    }
-
-	}
-
-	unlink(f2);
-	free(f2);
-	tmp = cut(f, pos, end);
-
-	mprintf("\nThe scanner was executed %d times.\n", exec);
-	mprintf("The signature length is %d (%d hex)\n", end - pos, 2 * (end - pos));
-
-	if(end - pos < MIN_LENGTH) {
-	    mprintf("\nWARNING: THE SIGNATURE IS TOO SMALL (PROBABLY ONLY A PART OF A REAL SIGNATURE).\n");
-	    mprintf("         PLEASE DON'T USE IT.\n\n");
-	}
-
-	if((fd = fopen(tmp, "rb")) == NULL) {
-	    mprintf("!Can't believe. Where is my signature, dude ?\n");
-	    exit(99);
-	}
-
-	signame = (char *) mcalloc(strlen(f) + 10, sizeof(char));
-	sprintf(signame, "%s.sig", f);
-	if(fileinfo(signame, 1) != -1) {
-	    mprintf("File %s exists.\n", signame);
-	    free(signame);
-	    signame = cl_gentemp(".");
-	}
-
-	bsigname = (char *) mcalloc(strlen(f) + 10, sizeof(char));
-	sprintf(bsigname, "%s.bsig", f);
-	if(fileinfo(bsigname, 1) != -1) {
-	    mprintf("File %s exists.\n", bsigname);
-	    free(bsigname);
-	    bsigname = cl_gentemp(".");
-	}
-
-	if((wd = fopen(signame, "wb")) == NULL) {
-	    mprintf("Can't write to %s\n", signame);
-	    unlink(tmp);
-	    free(tmp);
-	    exit(15);
-	}
-
-	mprintf("Saving signature in %s file.\n", signame);
-
-	while((bytes = fread(buffer, 1, FILEBUFF, fd)) > 0) {
-	    pt = cl_str2hex(buffer, bytes);
-	    fwrite(pt, 1, 2 * bytes, wd);
-	    free(pt);
-	}
-
-	mprintf("Saving binary signature in %s file.\n", bsigname);
-	rename(tmp, bsigname);
-
-	fclose(fd);
-	fclose(wd);
-	free(tmp);
-	free(signame);
-	free(bsigname);
+	help();
     }
 
-    /* free_opt(opt); */
 }
 
 int countlines(const char *filename)
@@ -478,49 +192,11 @@ int countlines(const char *filename)
     return lines;
 }
 
-const char *getdbdir(void)
-{
-        struct cfgstruct *copt, *cpt;
-	struct cl_cvd *d1, *d2;
-        const char *dbdir;
-
-
-    dbdir = cl_retdbdir();
-    if((copt = parsecfg(CONFDIR"/clamav.conf", 0))) {
-	if((cpt = cfgopt(copt, "DatabaseDirectory")) || (cpt = cfgopt(copt, "DataDirectory"))) {
-	    if(strcmp(cl_retdbdir(), cpt->strarg)) {
-		char *daily = (char *) mmalloc(strlen(cpt->strarg) + strlen(cl_retdbdir()) + 15);
-		sprintf(daily, "%s/daily.cvd", cpt->strarg);
-		if((d1 = cl_cvdhead(daily))) {
-		    sprintf(daily, "%s/daily.cvd", cl_retdbdir());
-		    if((d2 = cl_cvdhead(daily))) {
-			free(daily);
-			if(d1->version > d2->version)
-			    dbdir = cpt->strarg;
-			else
-			    dbdir = cl_retdbdir();
-			cl_cvdfree(d2);
-		    } else {
-			free(daily);
-			dbdir = cpt->strarg;
-		    }
-		    cl_cvdfree(d1);
-		} else {
-		    free(daily);
-		    dbdir = cl_retdbdir();
-		}
-	    }
-	}
-    }
-
-    return dbdir;
-}
-
 int build(struct optstruct *opt)
 {
 	int ret, no = 0, realno = 0, bytes, itmp;
 	struct stat foo;
-	char buffer[FILEBUFF], *tarfile = NULL, *gzfile = NULL, header[257],
+	char buffer[FILEBUFF], *tarfile = NULL, *gzfile = NULL, header[512],
 	     smbuff[30], *pt;
         struct cl_node *root = NULL;
 	FILE *tar, *cvd, *fd;
@@ -529,17 +205,13 @@ int build(struct optstruct *opt)
 	struct tm *brokent;
 	struct cl_cvd *oldcvd = NULL;
 
-    /* build a tar.gz archive
-     * we need: COPYING and {viruses.db, viruses.db2}+
-     * in current working directory
-     */
 
     if(stat("COPYING", &foo) == -1) {
 	mprintf("COPYING file not found in current working directory.\n");
 	exit(1);
     }
 
-    if(stat("viruses.db", &foo) == -1 && stat("viruses.db2", &foo) == -1) {
+    if(stat("main.db", &foo) == -1 && stat("daily.db", &foo) == -1 && stat("main.hdb", &foo) == -1 && stat("daily.hdb", &foo) == -1 && stat("main.ndb", &foo) == -1 && stat("daily.ndb", &foo) == -1) {
 	mprintf("Virus database not found in current working directory.\n");
 	exit(1);
     }
@@ -551,7 +223,7 @@ int build(struct optstruct *opt)
 	exit(1);
     }
 
-    cl_freetrie(root);
+    cl_free(root);
 
     mprintf("Database properly parsed.\n");
 
@@ -559,8 +231,7 @@ int build(struct optstruct *opt)
 	mprintf("WARNING: There are no signatures in the database(s).\n");
     } else {
 	mprintf("Signatures: %d\n", no);
-	realno = countlines("viruses.db") + countlines("viruses.db2");
-
+	realno = countlines("main.db") + countlines("daily.db") + countlines("main.hdb") + countlines("daily.hdb") + countlines("main.ndb") + countlines("daily.ndb");
 	if(realno != no) {
 	    mprintf("!Signatures in database: %d. Loaded: %d.\n", realno, no);
 	    mprintf("Please check the current directory and remove unnecessary databases\n");
@@ -569,7 +240,7 @@ int build(struct optstruct *opt)
 	}
     }
 
-    tarfile = cl_gentemp(".");
+    tarfile = cli_gentemp(".");
 
     switch(fork()) {
 	case -1:
@@ -577,7 +248,8 @@ int build(struct optstruct *opt)
 	    exit(1);
 	case 0:
 	    {
-		char *args[] = { "tar", "-cvf", tarfile, "COPYING", "viruses.db", "viruses.db2", "Notes", NULL };
+		char *args[] = { "tar", "-cvf", NULL, "COPYING", "main.db", "daily.db", "Notes", "viruses.db3", "main.hdb", "daily.hdb", "main.ndb", "daily.ndb", NULL };
+		args[2] = tarfile;
 		execv("/bin/tar", args);
 		mprintf("!Can't execute tar\n");
 		perror("tar");
@@ -597,7 +269,7 @@ int build(struct optstruct *opt)
 	exit(1);
     }
 
-    gzfile = cl_gentemp(".");
+    gzfile = cli_gentemp(".");
     if((gz = gzopen(gzfile, "wb")) == NULL) {
 	mprintf("!Can't open file %s to write.\n", gzfile);
 	exit(1);
@@ -614,7 +286,7 @@ int build(struct optstruct *opt)
 
 
     /* try to read cvd header of old database */
-    sprintf(buffer, "%s/%s", getdbdir(), getargc(opt, 'b'));
+    sprintf(buffer, "%s/%s", freshdbdir(), getargc(opt, 'b'));
     if((oldcvd = cl_cvdhead(buffer)) == NULL)
 	mprintf("WARNING: CAN'T READ CVD HEADER OF CURRENT DATABASE %s\n", buffer);
 
@@ -655,20 +327,21 @@ int build(struct optstruct *opt)
     strcat(header, smbuff);
 
     /* MD5 */
-    pt = cl_md5file(gzfile);
+    pt = cli_md5file(gzfile);
     strcat(header, pt);
     free(pt);
     strcat(header, ":");
 
-    /* builder - question */
+    /* ask for builder name */
     fflush(stdin);
     mprintf("Builder id: ");
     fscanf(stdin, "%s", smbuff);
 
     /* digital signature */
     fd = fopen(gzfile, "rb");
-    __md5_stream(fd, &buffer);
+    pt = cli_md5stream(fd, buffer);
     fclose(fd);
+    free(pt);
     if(!(pt = getdsig(getargl(opt, "server"), smbuff, buffer))) {
 	mprintf("No digital signature - no CVD file...\n");
 	unlink(gzfile);
@@ -679,9 +352,12 @@ int build(struct optstruct *opt)
     free(pt);
     strcat(header, ":");
 
-    /* builder - add */
+    /* add builder */
     strcat(header, smbuff);
 
+    /* add current time */
+    sprintf(header + strlen(header), ":%d", (int) timet);
+
     /* fill up with spaces */
 
     while(strlen(header) < 512)
@@ -819,29 +495,31 @@ char *getdsig(const char *host, const char *user, const char *data)
 
 int unpack(struct optstruct *opt)
 {
-	FILE *fd;
+	int fd;
 	char *name;
 
     if(optl(opt, "unpack-current")) {
-	name = mcalloc(300, sizeof(char)); /* FIXME */
-	sprintf(name, "%s/%s", getdbdir(), getargl(opt, "unpack-current"));
+	name = mcalloc(strlen(freshdbdir()) + strlen(getargl(opt, "unpack-current")) + 2, sizeof(char));
+	sprintf(name, "%s/%s", freshdbdir(), getargl(opt, "unpack-current"));
     } else
-	name = getargc(opt, 'u');
+	name = strdup(getargc(opt, 'u'));
 
-    if((fd = fopen(name, "rb")) == NULL) {
+    if((fd = open(name, O_RDONLY|O_BINARY)) == -1) {
 	mprintf("!Can't open CVD file %s\n", name);
+	free(name);
 	exit(1);
     }
 
-    fseek(fd, 512L, SEEK_SET);
+    free(name);
+    lseek(fd, 512, SEEK_SET);
 
-    if(cli_untgz(fileno(fd), ".")) {
+    if(cli_untgz(fd, ".")) {
 	mprintf("!Can't unpack file.\n");
-	fclose(fd);
+	close(fd);
 	exit(1);
     }
 
-    fclose(fd);
+    close(fd);
     exit(0);
 }
 
@@ -860,6 +538,7 @@ int listdb(const char *filename)
 
     if(!(buffer = (char *) mmalloc(FILEBUFF))) {
 	mprintf("!listdb(): Can't allocate memory.\n");
+	fclose(fd);
 	return -1;
     }
 
@@ -880,7 +559,7 @@ int listdb(const char *filename)
 	    tmpdir = "/tmp";
 #endif
 
-	dir = cl_gentemp(tmpdir);
+	dir = cli_gentemp(tmpdir);
 	if(mkdir(dir, 0700)) {
 	    mprintf("!listdb(): Can't create temporary directory %s\n", dir);
 	    free(buffer);
@@ -894,7 +573,7 @@ int listdb(const char *filename)
 
 	/* start */
 
-	tmp = cl_gentemp(tmpdir);
+	tmp = cli_gentemp(tmpdir);
 	if((tmpd = fopen(tmp, "wb+")) == NULL) {
 	    mprintf("!listdb(): Can't create temporary file %s\n", tmp);
 	    free(dir);
@@ -917,6 +596,7 @@ int listdb(const char *filename)
 	    mprintf("!listdb(): Can't unpack CVD file.\n");
 	    cli_rmdirs(dir);
 	    free(dir);
+	    fclose(tmpd);
 	    unlink(tmp);
 	    free(tmp);
 	    free(buffer);
@@ -938,26 +618,69 @@ int listdb(const char *filename)
 	return 0;
     }
 
+    if(cli_strbcasestr(filename, ".db") || cli_strbcasestr(filename, ".db2")) {
+	/* old style database */
 
-    /* old style database */
+	while(fgets(buffer, FILEBUFF, fd)) {
+	    line++;
+	    pt = strchr(buffer, '=');
+	    if(!pt) {
+		mprintf("!listdb(): Malformed pattern line %d (file %s).\n", line, filename);
+		fclose(fd);
+		free(buffer);
+		return -1;
+	    }
 
-    while(fgets(buffer, FILEBUFF, fd)) {
-	line++;
-	pt = strchr(buffer, '=');
-	if(!pt) {
-	    mprintf("!listdb(): Malformed pattern line %d (file %s).\n", line, filename);
-	    fclose(fd);
-	    free(buffer);
-	    return -1;
-	}
-
-	start = buffer;
-	*pt = 0;
-
-	if((pt = strstr(start, " (Clam)")))
+	    start = buffer;
 	    *pt = 0;
 
-	mprintf("%s\n", start);
+	    if((pt = strstr(start, " (Clam)")))
+		*pt = 0;
+
+	    mprintf("%s\n", start);
+	}
+
+    } else if(cli_strbcasestr(filename, ".hdb")) {
+
+	while(fgets(buffer, FILEBUFF, fd)) {
+	    line++;
+	    cli_chomp(buffer);
+	    start = cli_strtok(buffer, 2, ":");
+
+	    if(!start) {
+		mprintf("!listdb(): Malformed pattern line %d (file %s).\n", line, filename);
+		fclose(fd);
+		free(buffer);
+		return -1;
+	    }
+
+	    if((pt = strstr(start, " (Clam)")))
+		*pt = 0;
+
+	    mprintf("%s\n", start);
+	    free(start);
+	}
+
+    } else if(cli_strbcasestr(filename, ".ndb")) {
+
+	while(fgets(buffer, FILEBUFF, fd)) {
+	    line++;
+	    cli_chomp(buffer);
+	    start = cli_strtok(buffer, 0, ":");
+
+	    if(!start) {
+		mprintf("!listdb(): Malformed pattern line %d (file %s).\n", line, filename);
+		fclose(fd);
+		free(buffer);
+		return -1;
+	    }
+
+	    if((pt = strstr(start, " (Clam)")))
+		*pt = 0;
+
+	    mprintf("%s\n", start);
+	    free(start);
+	}
     }
 
     fclose(fd);
@@ -985,6 +708,8 @@ int listdir(const char *dirname)
 	    if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..") &&
 	    (cli_strbcasestr(dent->d_name, ".db")  ||
 	     cli_strbcasestr(dent->d_name, ".db2") ||
+	     cli_strbcasestr(dent->d_name, ".hdb") ||
+	     cli_strbcasestr(dent->d_name, ".ndb") ||
 	     cli_strbcasestr(dent->d_name, ".cvd"))) {
 
 		dbfile = (char *) mcalloc(strlen(dent->d_name) + strlen(dirname) + 2, sizeof(char));
@@ -1021,7 +746,7 @@ void listsigs(struct optstruct *opt)
     if((name = getargc(opt, 'l')))
 	ret = listdb(name);
     else
-	ret = listdir(getdbdir());
+	ret = listdir(freshdbdir());
 
     ret ? exit(1) : exit(0);
 }
@@ -1037,12 +762,11 @@ void help(void)
     mprintf("    --quiet                                be quiet, output only error messages\n");
     mprintf("    --debug                                enable debug messages\n");
     mprintf("    --stdout                               write to stdout instead of stderr\n");
-    mprintf("                                           (this help is always written to stdout)\n");
     mprintf("    --hex-dump                             convert data from stdin to a hex\n");
     mprintf("                                           string and print it on stdout\n");
-    mprintf("    --command              -c              scanner command string, with options\n");
-    mprintf("    --string               -s              'virus found' string in scan. output\n");
-    mprintf("    --file                 -f              infected file\n");
+    mprintf("    --md5 [FILES]                          generate MD5 checksum from stdin\n");
+    mprintf("                                           or MD5 sigs for FILES\n");
+    mprintf("    --html-normalise=FILE                  create normalised parts of HTML file\n");
     mprintf("    --info=FILE            -i FILE         print database information\n");
     mprintf("    --build=NAME           -b NAME         build a CVD file\n");
     mprintf("    --server=ADDR                          ClamAV Signing Service address\n");
